'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.process = undefined;

var _sortBy2 = require('lodash/sortBy');

var _sortBy3 = _interopRequireDefault(_sortBy2);

var _omit2 = require('lodash/omit');

var _omit3 = _interopRequireDefault(_omit2);

var _isString2 = require('lodash/isString');

var _isString3 = _interopRequireDefault(_isString2);

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _isEmpty2 = require('lodash/isEmpty');

var _isEmpty3 = _interopRequireDefault(_isEmpty2);

var _setQuickType = require('./setQuickType');

var setQuickType = _interopRequireWildcard(_setQuickType);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function renderGemo(chart, gemo) {
    switch (gemo) {
        case 'line':
            chart = chart.line();
            break;
        case 'area':
            chart = chart.area();
            break;
        case 'bar':
        case 'interval':
            chart = chart.interval();
            break;
        case 'point':
            chart = chart.point();
            break;
        case 'schema':
            chart = chart.schema();
            break;
        case 'polygon':
            chart = chart.polygon();
            break;
        case 'contour':
            chart = chart.contour();
            break;
        case 'heatmap':
            chart = chart.heatmap();
            break;
        case 'edge':
            chart = chart.edge();
            break;
        default:
            chart = chart.line();
    }
    return chart;
}
function skipPosition(dataMapping, currSeries) {
    if (!currSeries.position && dataMapping.column && dataMapping.row) {
        if (currSeries.quickType === 'pie') {
            currSeries.position = "" + dataMapping.row[0];
        } else {
            currSeries.position = dataMapping.column[0] + "*" + dataMapping.row[0];
        }
    }
    return currSeries;
}
function setSeriesPosition(chart, currSeries) {
    return chart.position(currSeries.position);
}
function setSeriesAdjust(chart, currSeries) {
    var adjust = currSeries.adjust;
    if (!(0, _isEmpty3.default)(adjust)) {
        return chart.adjust(adjust);
    }
    return chart;
}
function setSeriesShape(chart, dataMapping, currSeries) {
    var dim = dataMapping.shape;
    var shape = currSeries.shape;
    if (dim && shape) {
        return chart.shape(dim, shape);
    } else if (dim && !shape) {
        return chart.shape(dim);
    } else if (shape) {
        return chart.shape(shape);
    }
    return chart;
}
function setSeriesColor(chart, dataMapping, currSeries) {
    var color = (0, _get3.default)(currSeries, 'color');
    var dim = dataMapping.color;
    if (dim && color) {
        return chart.color(dim, color);
    } else if (dim && !color) {
        return chart.color(dim);
    } else if (color) {
        return chart.color(color);
    }
    return chart;
}
function setSeriesSize(chart, dataMapping, currSeries) {
    var dim = dataMapping.size;
    var size = currSeries.size;
    if (dim && size) {
        return chart.size(dim, size);
    } else if (dim && !size) {
        return chart.size(dim);
    } else if (size) {
        return chart.size(size);
    }
    return chart;
}
function setSeriesOpacity(chart, dataMapping, currSeries) {
    var dim = dataMapping.opacity;
    var opacity = currSeries.opacity;
    if (dim && opacity) {
        return chart.opacity(dim, opacity);
    } else if (dim && !opacity) {
        return chart.opacity(dim);
    } else if (opacity) {
        return chart.opacity(opacity);
    }
    return chart;
}
function setSeriesLabel(chart, currSeries) {
    var label = currSeries.label;
    if (!(0, _isEmpty3.default)(label)) {
        if ((0, _isString3.default)(label)) {
            return chart.label(label);
        }
        if (label.dataKey) {
            var options = (0, _omit3.default)(label, 'dataKey');
            return chart.label(label.dataKey, options);
        }
    }
    return chart;
}
function setSeriesStyle(chart, currSeries) {
    var style = currSeries.style;
    if (!(0, _isEmpty3.default)(style)) {
        var options = (0, _omit3.default)(style, 'dataKey');
        if (style.dataKey) {
            return chart.style(style.dataKey, options);
        } else {
            return chart.style(options);
        }
    }
    return chart;
}
function setSeriesTooltip(chart, currSeries) {
    var tooltip = currSeries.tooltip;
    if (typeof tooltip === 'boolean') {
        return chart.tooltip(false);
    }
    if (!(0, _isEmpty3.default)(tooltip)) {
        if ((0, _isString3.default)(tooltip)) {
            return chart.tooltip(tooltip);
        }
        if (tooltip.dataKey && tooltip.callback) {
            return chart.tooltip(tooltip.dataKey, tooltip.callback);
        }
        if (tooltip.dataKey) {
            return chart.tooltip(tooltip.dataKey);
        }
    }
    return chart;
}
function setSeriesSelect(chart, currSeries) {
    var select = currSeries.select;
    if (select === false) {
        return chart.select(false);
    }
    if (select) {
        if (select === true) {
            return chart.select(true);
        }
        return chart.select(true, select);
    }
    return chart;
}
var process = exports.process = function process(chart, config) {
    if ((0, _isEmpty3.default)(config.series)) {
        return chart;
    }
    config.series = Array.isArray(config.series) ? config.series : [config.series];
    config = setQuickType.process(config);
    var dataMapping = config.dataMapping;
    var series = config.series;
    series = (0, _sortBy3.default)(series, 'index');
    var chartInstance;
    series.forEach(function (currSeries) {
        currSeries = skipPosition(dataMapping, currSeries);
        chartInstance = renderGemo(chart, currSeries.gemo);
        chartInstance = setSeriesPosition(chartInstance, currSeries);
        chartInstance = setSeriesAdjust(chartInstance, currSeries);
        chartInstance = setSeriesShape(chartInstance, dataMapping, currSeries);
        chartInstance = setSeriesColor(chartInstance, dataMapping, currSeries);
        chartInstance = setSeriesOpacity(chartInstance, dataMapping, currSeries);
        chartInstance = setSeriesSize(chartInstance, dataMapping, currSeries);
        chartInstance = setSeriesLabel(chartInstance, currSeries);
        chartInstance = setSeriesTooltip(chartInstance, currSeries);
        chartInstance = setSeriesStyle(chartInstance, currSeries);
        chartInstance = setSeriesSelect(chartInstance, currSeries);
    });
    return chartInstance;
};
//# sourceMappingURL=setSeriesConfig.js.map