'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _isEqual2 = require('lodash/isEqual');

var _isEqual3 = _interopRequireDefault(_isEqual2);

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _isFunction2 = require('lodash/isFunction');

var _isFunction3 = _interopRequireDefault(_isFunction2);

var _omit2 = require('lodash/omit');

var _omit3 = _interopRequireDefault(_omit2);

var _isEmpty2 = require('lodash/isEmpty');

var _isEmpty3 = _interopRequireDefault(_isEmpty2);

var _isPlainObject2 = require('lodash/isPlainObject');

var _isPlainObject3 = _interopRequireDefault(_isPlainObject2);

var _cloneDeep2 = require('lodash/cloneDeep');

var _cloneDeep3 = _interopRequireDefault(_cloneDeep2);

var _loadShapes = require('../shapes/loadShapes');

var _loadShapes2 = _interopRequireDefault(_loadShapes);

var _DataSetUtils = require('../utils/DataSetUtils');

var DataSetUtils = _interopRequireWildcard(_DataSetUtils);

var _setCoordConfig = require('../components/setCoordConfig');

var setCoordConfig = _interopRequireWildcard(_setCoordConfig);

var _setAxisConfig = require('../components/setAxisConfig');

var setAxisConfig = _interopRequireWildcard(_setAxisConfig);

var _setSeriesConfig = require('../components/setSeriesConfig');

var setSeriesConfig = _interopRequireWildcard(_setSeriesConfig);

var _setDataMappingConfig = require('../components/setDataMappingConfig');

var setDataMappingConfig = _interopRequireWildcard(_setDataMappingConfig);

var _setLengendConfig = require('../components/setLengendConfig');

var setLengendConfig = _interopRequireWildcard(_setLengendConfig);

var _setGuideConfig = require('../components/setGuideConfig');

var setGuideConfig = _interopRequireWildcard(_setGuideConfig);

var _setTooltipConfig = require('../components/setTooltipConfig');

var setTooltipConfig = _interopRequireWildcard(_setTooltipConfig);

var _setScaleConfig = require('../components/setScaleConfig');

var setScaleConfig = _interopRequireWildcard(_setScaleConfig);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var G2 = require('@antv/g2');
var CommonChart = function () {
    function CommonChart(config) {
        this.viewInstance = {};
        this.config = (0, _cloneDeep3.default)(config);
        this.config = this.checkChartConfig(this.config);
        var chart = this.chartInstance = new G2.Chart(this.config.chart);
    }
    CommonChart.prototype.checkChartConfig = function (config) {
        var chart = config.chart;
        if (!chart || !chart.height) {
            throw new Error('please set correct chart option');
        }
        return config;
    };
    ;
    CommonChart.prototype.destroy = function (chart) {
        chart && chart.destroy();
    };
    CommonChart.prototype.clear = function (chart) {
        chart && chart.clear();
    };
    CommonChart.prototype.createView = function (chart, config) {
        var view = chart.view();
        if (!config.viewId) {
            throw new Error('you must set viewId');
        }
        this.viewInstance[config.viewId] = view;
        return view;
    };
    CommonChart.prototype.setEvents = function (chart, config) {
        var chartConfig = config.chart;
        var events = Object.keys(chartConfig).filter(function (entry) {
            return (/^on/.test(entry)
            );
        });
        var eventsRes = {};
        events.forEach(function (entry) {
            var eventName = entry.slice(2, entry.length);
            var eventLowerCase = eventName.toLowerCase();
            var content = chartConfig[entry];
            if ((0, _isPlainObject3.default)(content)) {
                for (var res in content) {
                    if (content.hasOwnProperty(res)) {
                        chart.on(res + ":" + eventLowerCase, content[res]);
                    }
                }
            } else {
                chart.on(eventLowerCase, content);
            }
        });
    };
    CommonChart.prototype.setDataSource = function (chart, data) {
        chart.source(data);
    };
    CommonChart.prototype.setScale = function (chart, config) {
        return setScaleConfig.process(chart, config);
    };
    CommonChart.prototype.setCoord = function (chart, config) {
        return setCoordConfig.process(chart, config.coord);
    };
    CommonChart.prototype.setSeries = function (chart, config) {
        return setSeriesConfig.process(chart, config);
    };
    CommonChart.prototype.setAxis = function (chart, config) {
        return setAxisConfig.process(chart, config);
    };
    CommonChart.prototype.setTooltip = function (chart, config) {
        return setTooltipConfig.process(chart, config);
    };
    CommonChart.prototype.setGuide = function (chart, config) {
        return setGuideConfig.process(chart, config);
    };
    CommonChart.prototype.setLegend = function (chart, config) {
        return setLengendConfig.process(chart, config);
    };
    CommonChart.prototype.setContent = function (chart, config) {
        this.setScale(chart, config);
        this.setAxis(chart, config);
        this.setSeries(chart, config);
        this.setCoord(chart, config);
        this.setTooltip(chart, config);
        this.setGuide(chart, config);
    };
    CommonChart.prototype.setView = function (item, chart, config) {
        item = setDataMappingConfig.process(item);
        var view = this.createView(chart, item);
        var viewData = item.data;
        if (item.data) {
            viewData = DataSetUtils.preprocessing(item.data, item.dataPre);
        } else if (!item.data && !item.dataPre) {
            viewData = config.calData;
        } else if (!item.data && item.dataPre) {
            viewData = DataSetUtils.preprocessing(config.data, item.dataPre);
        }
        var finalData = viewData;
        if (item.dataView) {
            finalData = viewData[item.dataView];
        }
        this.setDataSource(view, finalData);
        this.setContent(view, item);
        return view;
    };
    CommonChart.prototype.setViews = function (chart, config) {
        var views = config.views;
        if ((0, _isEmpty3.default)(views)) {
            return;
        }
        views = Array.isArray(views) ? views : [views];
        for (var _i = 0, views_1 = views; _i < views_1.length; _i++) {
            var item = views_1[_i];
            this.setView(item, chart, config);
        }
    };
    CommonChart.prototype.setFacetViews = function (chart, facet, views, dataMapping) {
        var viewData = facet.data;
        if (!views.dataMapping) {
            views.dataMapping = dataMapping;
        } else {
            views = setDataMappingConfig.process(views);
        }
        if (views.dataPre) {
            viewData = DataSetUtils.preprocessing(viewData, views.dataPre);
        }
        this.setDataSource(chart, viewData);
        this.setContent(chart, views);
    };
    CommonChart.prototype.setFacet = function (chart, config) {
        var _this = this;
        var facet = config.facet;
        var dataMapping = config.dataMapping;
        if (!facet) {
            return;
        }
        var options = (0, _omit3.default)(facet, ['type', 'views']);
        if ((0, _isEmpty3.default)(facet.views) && !(0, _isFunction3.default)(facet.views)) {
            return chart.facet(facet.type, options);
        }
        if ((0, _isFunction3.default)(facet.views)) {
            options.eachView = function (v, f) {
                var options = facet.views(v, f);
                _this.setFacetViews(v, f, options, dataMapping);
            };
        } else {
            facet.views = Array.isArray(facet.views) ? facet.views : [facet.views];
            options.eachView = function (v, f) {
                _this.setFacetViews(v, f, facet.views[0], dataMapping);
            };
        }
        return chart.facet(facet.type, options);
    };
    CommonChart.prototype.render = function () {
        var config = setDataMappingConfig.process(this.config);
        var data = config.data,
            dataMapping = config.dataMapping,
            dataPre = config.dataPre;
        var chart = this.chartInstance;
        (0, _loadShapes2.default)();
        this.setEvents(chart, config);
        config.calData = DataSetUtils.preprocessing(data, dataPre);
        var finalData = config.calData;
        if (config.dataView) {
            finalData = finalData[config.dataView];
        }
        if (config.viewId) {
            var view = this.createView(chart, config);
            this.setDataSource(view, finalData);
            this.setContent(view, config);
        } else {
            this.setDataSource(chart, finalData);
            this.setContent(chart, config);
        }
        this.setLegend(chart, config);
        this.setViews(chart, config);
        this.setFacet(chart, config);
        this.oriConfig = config;
        chart.render();
    };
    CommonChart.prototype.repaintWidthHeight = function (config) {
        var oriConfig = this.oriConfig;
        var chart = this.chartInstance;
        var width = (0, _get3.default)(config, 'chart.width');
        if (width && !(0, _isEqual3.default)((0, _get3.default)(oriConfig, 'chart.width'), width)) {
            chart.changeWidth(width);
        }
        var height = (0, _get3.default)(config, 'chart.height');
        if (height && !(0, _isEqual3.default)((0, _get3.default)(oriConfig, 'chart.height'), height)) {
            chart.changeHeight(height);
        }
    };
    CommonChart.prototype.repaintData = function (chart, oriConfig, config) {
        if (!(0, _isEmpty3.default)(config.data) && !(0, _isEqual3.default)(oriConfig.data, config.data)) {
            config.calData = DataSetUtils.preprocessing(config.data, config.dataPre);
            var finalData = config.calData;
            if (config.dataView) {
                finalData = finalData[config.dataView];
            }
            chart.changeData(finalData);
        } else {
            config.calData = oriConfig.calData;
        }
    };
    CommonChart.prototype.repaintContent = function (chart, oriConfig, config) {
        config = setDataMappingConfig.process(config);
        this.repaintData(chart, oriConfig, config);
        if (config.dataMapping && !(0, _isEqual3.default)(oriConfig.dataMapping, config.dataMapping)) {
            this.setScale(chart, config);
        }
        if (config.coord && !(0, _isEqual3.default)(oriConfig.coord, config.coord)) {
            this.setCoord(chart, config);
        }
        if (config.axis && !(0, _isEqual3.default)(oriConfig.axis, config.axis)) {
            this.setAxis(chart, config);
        }
        if (config.dataMapping && !(0, _isEqual3.default)(oriConfig.dataMapping, config.dataMapping) || config.series && !(0, _isEqual3.default)(oriConfig.series, config.series)) {
            this.setSeries(chart, config);
        }
        if (config.tooltip && !(0, _isEqual3.default)(oriConfig.tooltip, config.tooltip)) {
            this.setTooltip(chart, config);
        }
        if (config.guide && !(0, _isEqual3.default)(oriConfig.guide, config.guide)) {
            this.setGuide(chart, config);
        }
    };
    CommonChart.prototype.repaintViews = function (chart, oriConfig, config) {
        var viewsConfig = config.views;
        var oViewsConfig = oriConfig.views;
        if (!(0, _isEqual3.default)(oriConfig.views, config.views)) {
            var _loop_1 = function _loop_1(item) {
                var oriView = oViewsConfig.filter(function (res) {
                    return res.viewId === item.viewId;
                });
                if (oriView.length) {
                    var view = this_1.viewInstance[item.viewId];
                    this_1.repaintContent(view, oriView[0], item);
                    view.repaint();
                } else {
                    var view = this_1.setView(item, chart, config);
                    view.repaint();
                }
            };
            var this_1 = this;
            for (var _i = 0, viewsConfig_1 = viewsConfig; _i < viewsConfig_1.length; _i++) {
                var item = viewsConfig_1[_i];
                _loop_1(item);
            }
        }
    };
    CommonChart.prototype.renderDiffConfig = function (config) {
        var oriConfig = this.oriConfig;
        var viewsConfig = config.views;
        var chart = this.chartInstance;
        var hasChartChange = false;
        config = this.checkChartConfig(config);
        this.repaintWidthHeight(config);
        if (!config.viewId) {
            this.repaintContent(chart, oriConfig, config);
            hasChartChange = true;
        } else if (config.viewId && !oriConfig.viewId) {
            var view = this.setView(config, chart, config);
            view.repaint();
        } else if (config.viewId && oriConfig.viewId && (0, _isEqual3.default)(oriConfig.viewId, config.viewId)) {
            var view = this.viewInstance[config.viewId];
            this.repaintContent(view, oriConfig, config);
            view.repaint();
        }
        this.repaintViews(chart, oriConfig, config);
        if (config.legend && !(0, _isEqual3.default)(oriConfig.legend, config.legend)) {
            this.setLegend(chart, config);
            hasChartChange = true;
        }
        if (config.facet && !(0, _isEqual3.default)(oriConfig.facet, config.facet)) {
            this.setFacet(chart, config);
            hasChartChange = true;
        }
        if (hasChartChange) {
            chart.repaint();
        }
    };
    CommonChart.prototype.repaint = function (config) {
        if ((0, _isEmpty3.default)(config)) {
            return;
        }
        config = (0, _cloneDeep3.default)(config);
        this.renderDiffConfig(config);
        this.oriConfig = config;
    };
    CommonChart.prototype.getWidth = function () {
        return this.chartInstance.get('width');
    };
    CommonChart.prototype.getHeight = function () {
        return this.chartInstance.get('height');
    };
    return CommonChart;
}();
exports.default = CommonChart;
//# sourceMappingURL=CommonChart.js.map